﻿//Author:      night-king
// Email:       deepleo@163.com
//Home:       http://deepleo.com
//Github:     https://github.com/night-king/
//Date:         2013-11-1 
//Place:        Wuhan@China

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Deepleo.LazyScheduler.Setting;

namespace Deepleo.LazyScheduler.Sample
{
    public partial class Form1 : Form
    {
        private Scheduler _scheduler = null;
        public Form1()
        {
            InitializeComponent();
        }
        private void buttonStart_Click(object sender, EventArgs e)
        {
            if (_scheduler.Status == SchedulerStatus.Stopped)
            {
                this.rtbLog.Clear();
                var excludeWeeks = new List<DayOfWeek>();
                if (checkBoxWeek.Checked)
                {
                    if (!checkBoxMonday.Checked) excludeWeeks.Add(DayOfWeek.Monday);
                    if (!checkBoxTuesday.Checked) excludeWeeks.Add(DayOfWeek.Tuesday);
                    if (!checkBoxWednesday.Checked) excludeWeeks.Add(DayOfWeek.Wednesday);
                    if (!checkBoxThursday.Checked) excludeWeeks.Add(DayOfWeek.Thursday);
                    if (!checkBoxFriday.Checked) excludeWeeks.Add(DayOfWeek.Friday);
                    if (!checkBoxSaturday.Checked) excludeWeeks.Add(DayOfWeek.Saturday);
                    if (!checkBoxSunday.Checked) excludeWeeks.Add(DayOfWeek.Sunday);
                }
                var settings = new List<ISchedulerSetting>();
                if (checkBoxInterval.Checked)
                {
                    var interval = new IntervalSchedulerSetting(excludeWeeks);
                    interval.Interval = new TimeSpan((int)(this.nudHours.Value), (int)(this.nudMinutes.Value), (int)(this.numSeconds.Value));
                    settings.Add(interval);
                }
                if (checkBoxFixTime.Checked)
                {
                    var fixTime = new FixTimeSchedulerSetting(excludeWeeks);
                    fixTime.Hour = dtpFixTime.Value.Hour;
                    fixTime.Minutes = dtpFixTime.Value.Minute;
                    fixTime.Second = dtpFixTime.Value.Second;
                    settings.Add(fixTime);
                }
                if (settings.Count == 0)
                {
                    MessageBox.Show("please config settings.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                _scheduler.Start(settings); 
                buttonStart.Text = "Stop";
                buttonSuspend.Text = "Suspend";
                buttonSuspend.Enabled = true;
            }
            else if (_scheduler.Status == SchedulerStatus.Running)
            {
                _scheduler.Stop();
                buttonStart.Text = "Start";
                buttonStart.Enabled = true;
                buttonSuspend.Enabled = false;
            }
        }

        private void buttonSuspend_Click(object sender, EventArgs e)
        {
            if (_scheduler.Status == SchedulerStatus.Running)
            {
                _scheduler.Suspend();
                buttonSuspend.Text = "Resume";
                buttonStart.Enabled = false;
                buttonSuspend.Enabled = true;
            }
            else if (_scheduler.Status == SchedulerStatus.Suspended)
            {
                _scheduler.Resume();
                buttonStart.Text = "Stop"; buttonSuspend.Text = "Suspend";
                buttonStart.Enabled = true;
                buttonSuspend.Enabled = true;
            }
        }

        private void checkBoxInterval_CheckedChanged(object sender, EventArgs e)
        {
            panelInterval.Enabled = checkBoxInterval.Checked;
        }

        private void checkBoxFixTime_CheckedChanged(object sender, EventArgs e)
        {
            panelFixTime.Enabled = checkBoxFixTime.Checked;
        }

        private void checkBoxWeek_CheckedChanged(object sender, EventArgs e)
        {
            panelWeek.Enabled = checkBoxWeek.Checked;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            checkBoxFixTime.Checked = false;
            checkBoxInterval.Checked = true;
            checkBoxWeek.Checked = true;
            panelFixTime.Enabled = false;
            buttonSuspend.Enabled = false;

            _scheduler = new Scheduler("my scheduler");

            _scheduler.HittedSuccessEvent += delegate
            {
                rtbLog.BeginInvoke(new Action(() =>
                {
                    rtbLog.AppendText(string.Format("hitted in: {0}{1}", DateTime.Now, Environment.NewLine));
                    rtbLog.ScrollToCaret();
                }));
            };
            _scheduler.FinishedEvent += delegate
            {
                rtbLog.BeginInvoke(new Action(() =>
                {
                    rtbLog.AppendText(string.Format("scheduler finished in: {0}{1}", DateTime.Now, Environment.NewLine));
                    rtbLog.ScrollToCaret();
                }));
            };
        }
    }
}
