﻿//Author:      night-king
// Email:       deepleo@163.com
//Home:       http://deepleo.com
//Github:     https://github.com/night-king/
//Date:         2013-11-1 
//Place:        Wuhan@China

using System;
using System.Collections.Generic;
using System.Text;
using Deepleo.LazyScheduler.Setting;
using Deepleo.LazyScheduler.Task;

namespace Deepleo.LazyScheduler
{
    public enum SchedulerStatus
    {
        Running = 1,
        Suspended = 2,
        Stopped = 3
    }
    public interface IScheduler
    {
        /// <summary>
        /// scheduler's name
        /// </summary>
        string Name
        {
            set;
            get;
        }

        /// <summary>
        /// scheduler's status
        /// </summary>
        SchedulerStatus Status
        {
            set;
            get;
        }

        /// <summary>
        ///  fired once when hit success once
        /// </summary>
        event EventHandler<EventArgs> HittedSuccessEvent;

        /// <summary>
        /// fired when this scheduler finished
        /// </summary>
        event EventHandler<EventArgs> FinishedEvent;

        /// <summary>
        /// Start this scheduler
        /// </summary>
        /// <param name="settings">scheduler's settings</param>
        void Start(IList<ISchedulerSetting> settings);

        /// <summary>
        /// Start this scheduler
        /// </summary>
        /// <param name="taskManager">the task collection's manager</param>
        /// <param name="settings">scheduler's settings</param>
        /// <param name="isSyncRunTask">is sync run task</param>
        void Start(ITaskManager taskManager, IList<ISchedulerSetting> settings, bool isSyncRunTask);

        /// <summary>
        /// stop this scheduler
        /// </summary>
        void Stop();

        /// <summary>
        /// suspend this scheduler
        /// </summary>
        void Suspend();

        /// <summary>
        /// Resume this scheduler
        /// </summary>
        void Resume();

    }
}
