﻿//Author:      night-king
// Email:       deepleo@163.com
//Home:       http://deepleo.com
//Github:     https://github.com/night-king/
//Date:         2013-11-1 
//Place:        Wuhan@China

using System;
using System.Collections.Generic;
using System.Text;
using Deepleo.LazyScheduler.Setting;
using Deepleo.LazyScheduler.Task;
using System.Threading;

namespace Deepleo.LazyScheduler
{
    /// <summary>
    /// Default scheduler
    /// </summary>
    public class Scheduler : IScheduler
    {
        public string Name
        {
            set;
            get;
        }

        public SchedulerStatus Status
        {
            get;
            set;
        }

        public IList<ISchedulerSetting> Settings
        {
            set;
            get;
        }

        public ITaskManager TaskManager
        {
            set;
            get;
        }
        public bool IsSyncRunTask
        {
            set;
            get;
        }

        public event EventHandler<EventArgs> HittedSuccessEvent;
        public event EventHandler<EventArgs> FinishedEvent;
        public Scheduler(string name)
        {
            Name = name;
            Status = SchedulerStatus.Stopped;
        }

        private Thread _workThread = null;
        public void Start(IList<ISchedulerSetting> settings)
        {
            Settings = settings;
            if (Status != SchedulerStatus.Stopped) throw new Exception(string.Format("can not start the scheduler because the current status of Scheduler is {0}. ", Status));
            _workThread = new Thread(new ThreadStart(doWork));
            _workThread.Start();
            Status = SchedulerStatus.Running;
        }

        public void Start(ITaskManager taskManager, IList<ISchedulerSetting> setting, bool isSyncRunTask)
        {
            TaskManager = taskManager;
            Settings = setting;
            if (Status != SchedulerStatus.Stopped) throw new Exception(string.Format("can not start the scheduler because the current status of Scheduler is {0}. ", Status));
            _workThread = new Thread(new ThreadStart(doWork));
            _workThread.Start();
            Status = SchedulerStatus.Running;
        }

        public void Stop()
        {
            if (Status != SchedulerStatus.Running && Status != SchedulerStatus.Suspended) return;
            _workThread.Abort();
            _workThread.Join();
            Status = SchedulerStatus.Stopped;
        }

        private ManualResetEvent _resetEvent = new ManualResetEvent(true);
        public void Suspend()
        {
            if (Status != SchedulerStatus.Running) throw new Exception(string.Format("can not suspend the scheduler because the current status of Scheduler is {0}. ", Status));
            _resetEvent.Reset();
            Status = SchedulerStatus.Suspended;
        }

        public void Resume()
        {
            if (Status != SchedulerStatus.Suspended) throw new Exception(string.Format("can not resume the scheduler because the current status of Scheduler is {0}. ", Status));
            _resetEvent.Set();
            Status = SchedulerStatus.Running;
        }

        private void doWork()
        {
            try
            {
                while (true)
                {
                    _resetEvent.WaitOne();
                    var timeSpan = Settings[0].CalculateNext();
                    for (int i = 1; i < Settings.Count; i++)
                    {
                        var setting = Settings[i];
                        var newTimeSpan = setting.CalculateNext();
                        if (newTimeSpan < timeSpan) timeSpan = newTimeSpan;
                    }
                    Thread.Sleep(timeSpan);
                    if (Status == SchedulerStatus.Stopped) break;
                    else
                    {
                        if (HittedSuccessEvent != null) HittedSuccessEvent(this, EventArgs.Empty);
                        if (TaskManager == null) continue;//if taskManager is not set,only fire hitted event.
                        var task = TaskManager.Next();
                        if (task == null) break;//if completeed all tasks, scheduler should be finish.
                        task.Execute(IsSyncRunTask, null);
                    }
                }
            }
            catch (ThreadAbortException) { }
            finally
            {
                if (FinishedEvent != null) FinishedEvent(this, EventArgs.Empty);
            }
        }


    }


}
